@@ -637,7 +637,31 @@ typedef enum SANITIZER_CBID_DRIVER_API_enum {
    SANITIZER_CBID_DRIVER_API_cuGraphExternalSemaphoresWaitNodeSetParams                   = 623,
    SANITIZER_CBID_DRIVER_API_cuGraphExecExternalSemaphoresSignalNodeSetParams             = 624,
    SANITIZER_CBID_DRIVER_API_cuGraphExecExternalSemaphoresWaitNodeSetParams               = 625,
-   SANITIZER_CBID_DRIVER_API_SIZE                                                         = 626,
+   SANITIZER_CBID_DRIVER_API_cuGetProcAddress                                             = 626,
+   SANITIZER_CBID_DRIVER_API_cuFlushGPUDirectRDMAWrites                                   = 627,
+   SANITIZER_CBID_DRIVER_API_cuGraphDebugDotPrint                                         = 628,
+   SANITIZER_CBID_DRIVER_API_cuStreamGetCaptureInfo_v2                                    = 629,
+   SANITIZER_CBID_DRIVER_API_cuStreamGetCaptureInfo_v2_ptsz                               = 630,
+   SANITIZER_CBID_DRIVER_API_cuStreamUpdateCaptureDependencies                            = 631,
+   SANITIZER_CBID_DRIVER_API_cuStreamUpdateCaptureDependencies_ptsz                       = 632,
+   SANITIZER_CBID_DRIVER_API_cuUserObjectCreate                                           = 633,
+   SANITIZER_CBID_DRIVER_API_cuUserObjectRetain                                           = 634,
+   SANITIZER_CBID_DRIVER_API_cuUserObjectRelease                                          = 635,
+   SANITIZER_CBID_DRIVER_API_cuGraphRetainUserObject                                      = 636,
+   SANITIZER_CBID_DRIVER_API_cuGraphReleaseUserObject                                     = 637,
+   SANITIZER_CBID_DRIVER_API_cuGraphAddMemAllocNode                                       = 638,
+   SANITIZER_CBID_DRIVER_API_cuGraphAddMemFreeNode                                        = 639,
+   SANITIZER_CBID_DRIVER_API_cuDeviceGraphMemTrim                                         = 640,
+   SANITIZER_CBID_DRIVER_API_cuDeviceGetGraphMemAttribute                                 = 641,
+   SANITIZER_CBID_DRIVER_API_cuDeviceSetGraphMemAttribute                                 = 642,
+   SANITIZER_CBID_DRIVER_API_cuGraphInstantiateWithFlags                                  = 643,
+   SANITIZER_CBID_DRIVER_API_cuDeviceGetExecAffinitySupport                               = 644,
+   SANITIZER_CBID_DRIVER_API_cuCtxCreate_v3                                               = 645,
+   SANITIZER_CBID_DRIVER_API_cuCtxGetExecAffinity                                         = 646,
+   SANITIZER_CBID_DRIVER_API_cuDeviceGetUuid_v2                                           = 647,
+   SANITIZER_CBID_DRIVER_API_cuGraphMemAllocNodeGetParams                                 = 648,
+   SANITIZER_CBID_DRIVER_API_cuGraphMemFreeNodeGetParams                                  = 649,
+   SANITIZER_CBID_DRIVER_API_SIZE                                                         = 650,
    SANITIZER_CBID_DRIVER_API_FORCE_INT                                                    = 0x7fffffff,
 } SANITIZER_CBID_DRIVER_API;
 
