@@ -1,6 +1,6 @@
 /* Gdb/Python header for private use by Python module.
 
-   Copyright (C) 2008-2016 Free Software Foundation, Inc.
+   Copyright (C) 2008-2020 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,8 +17,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef GDB_PYTHON_INTERNAL_H
-#define GDB_PYTHON_INTERNAL_H
+#ifndef PYTHON_PYTHON_INTERNAL_H
+#define PYTHON_PYTHON_INTERNAL_H
 
 #include "extension.h"
 #include "extension-priv.h"
@@ -27,14 +27,9 @@
    comes with the Python plugin for GCC.  See:
    https://gcc-python-plugin.readthedocs.org/en/latest/cpychecker.html
    The checker defines a WITH_ macro for each attribute it
-   exposes.  */
-
-#ifdef WITH_CPYCHECKER_RETURNS_BORROWED_REF_ATTRIBUTE
-#define CPYCHECKER_RETURNS_BORROWED_REF			\
-  __attribute__ ((cpychecker_returns_borrowed_ref))
-#else
-#define CPYCHECKER_RETURNS_BORROWED_REF
-#endif
+   exposes.  Note that we intentionally do not use
+   'cpychecker_returns_borrowed_ref' -- that idiom is forbidden in
+   gdb.  */
 
 #ifdef WITH_CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF_ATTRIBUTE
 #define CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF(ARG)		\
@@ -43,13 +38,6 @@
 #define CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF(ARG)
 #endif
 
-#ifdef WITH_CPYCHECKER_STEALS_REFERENCE_TO_ARG_ATTRIBUTE
-#define CPYCHECKER_STEALS_REFERENCE_TO_ARG(n) \
-   __attribute__ ((cpychecker_steals_reference_to_arg (n)))
-#else
-#define CPYCHECKER_STEALS_REFERENCE_TO_ARG(n)
-#endif
-
 #ifdef WITH_CPYCHECKER_SETS_EXCEPTION_ATTRIBUTE
 #define CPYCHECKER_SETS_EXCEPTION __attribute__ ((cpychecker_sets_exception))
 #else
@@ -63,8 +51,6 @@
 #define CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION
 #endif
 
-/* Python 2.4 doesn't include stdint.h soon enough to get {u,}intptr_t
-   needed by pyport.h.  */
 /* /usr/include/features.h on linux systems will define _POSIX_C_SOURCE
    if it sees _GNU_SOURCE (which config.h will define).
    pyconfig.h defines _POSIX_C_SOURCE to a different value than
@@ -85,6 +71,12 @@
 #define HAVE_SNPRINTF 1
 #endif
 
+/* Another kludge to avoid compilation errors because MinGW defines
+   'hypot' to '_hypot', but the C++ headers says "using ::hypot".  */
+#ifdef __MINGW32__
+# define _hypot hypot
+#endif
+
 /* Request clean size types from Python.  */
 #define PY_SSIZE_T_CLEAN
 
@@ -93,6 +85,7 @@
    from including our python/python.h header file.  */
 #include <Python.h>
 #include <frameobject.h>
+#include "py-ref.h"
 
 #if PY_MAJOR_VERSION >= 3
 #define IS_PY3K 1
@@ -103,8 +96,10 @@
 #define Py_TPFLAGS_CHECKTYPES 0
 
 #define PyInt_Check PyLong_Check
-#define PyInt_FromLong PyLong_FromLong
+#define PyInt_FromLong gdbpy_PyLong_FromLongLong
+#define PyInt_FromSsize_t PyLong_FromSsize_t
 #define PyInt_AsLong PyLong_AsLong
+#define PyInt_AsSsize_t PyLong_AsSsize_t
 
 #define PyString_FromString PyUnicode_FromString
 #define PyString_Decode PyUnicode_Decode
@@ -112,26 +107,6 @@
 #define PyString_Check PyUnicode_Check
 #endif
 
-#if HAVE_LIBPYTHON2_4
-/* Py_ssize_t is not defined until 2.5.
-   Logical type for Py_ssize_t is Py_intptr_t, but that fails in 64-bit
-   compilation due to several apparent mistakes in python2.4 API, so we
-   use 'int' instead.  */
-typedef int Py_ssize_t;
-#endif
-
-#ifndef PyVarObject_HEAD_INIT
-/* Python 2.4 does not define PyVarObject_HEAD_INIT.  */
-#define PyVarObject_HEAD_INIT(type, size)       \
-    PyObject_HEAD_INIT(type) size,
-
-#endif
-
-#ifndef Py_TYPE
-/* Python 2.4 does not define Py_TYPE.  */
-#define Py_TYPE(ob)             (((PyObject*)(ob))->ob_type)
-#endif
-
 /* If Python.h does not define WITH_THREAD, then the various
    GIL-related functions will not be defined.  However,
    PyGILState_STATE will be.  */
@@ -172,6 +147,13 @@ typedef unsigned long gdb_py_ulongest;
 typedef long Py_hash_t;
 #endif
 
+/* PyMem_RawMalloc appeared in Python 3.4.  For earlier versions, we can just
+   fall back to PyMem_Malloc.  */
+
+#if PY_VERSION_HEX < 0x03040000
+#define PyMem_RawMalloc PyMem_Malloc
+#endif
+
 /* Python 2.6 did not wrap Py_DECREF in 'do {...} while (0)', leading
    to 'suggest explicit braces to avoid ambiguous ‘else’' gcc errors.
    Wrap it ourselves, so that callers don't need to care.  */
@@ -179,15 +161,265 @@ typedef long Py_hash_t;
 static inline void
 gdb_Py_DECREF (void *op) /* ARI: editCase function */
 {
-  /* ... and Python 2.4 didn't cast OP to PyObject pointer on the
-     '(op)->ob_refcnt' references within the macro.  Cast it ourselves
-     too.  */
-  Py_DECREF ((PyObject *) op);
+  Py_DECREF (op);
 }
 
 #undef Py_DECREF
 #define Py_DECREF(op) gdb_Py_DECREF (op)
 
+/* PyObject_CallMethod's 'method' and 'format' parameters were missing
+   the 'const' qualifier before Python 3.4.  Hence, we wrap the
+   function in our own version to avoid errors with string literals.
+   Note, this is a variadic template because PyObject_CallMethod is a
+   varargs function and Python doesn't have a "PyObject_VaCallMethod"
+   variant taking a va_list that we could defer to instead.  */
+
+template<typename... Args>
+static inline PyObject *
+gdb_PyObject_CallMethod (PyObject *o, const char *method, const char *format,
+			 Args... args) /* ARI: editCase function */
+{
+  return PyObject_CallMethod (o,
+			      const_cast<char *> (method),
+			      const_cast<char *> (format),
+			      args...);
+}
+
+#undef PyObject_CallMethod
+#define PyObject_CallMethod gdb_PyObject_CallMethod
+
+/* CUDA-specific defines.  */
+
+extern int (*gdbpy_Arg_UnpackTuple) (PyObject *, const char *, Py_ssize_t, Py_ssize_t, ...);
+extern PyObject * (*_gdbpy_BuildValue_SizeT) (const char *, ...);
+extern PyObject * (*gdbpy_ErrFormat)(PyObject *, const char *, ...);
+extern PyObject * (*gdbpy_PySys_GetObject)(const char *);
+extern PyObject * (*gdbpy_PyObject_CallFunctionObjArgs) (PyObject *,...);
+extern PyObject * (*gdbpy_PyObject_CallMethodObjArgs) (PyObject *, PyObject *,...);
+extern PyObject * (*gdbpy_PyObject_CallMethod)(PyObject *o, const char *m, const char *format, ...);
+extern PyObject * (*gdbpy_PyErr_NewException) (const char *name, PyObject *base, PyObject *dict);
+
+extern int (*gdbpy_PyArg_ParseTuple) (PyObject *, const char *, ...);
+extern int (*gdbpy_PyObject_GetBuffer) (PyObject *, Py_buffer *, int);
+extern Py_ssize_t (*gdbpy_PyObject_Length)(PyObject *o);
+extern int (*gdbpy_PyArg_ParseTupleAndKeywords) (PyObject *obj, PyObject *, const char *, char **, ...);
+extern int (*gdbpy_PyArg_VaParseTupleAndKeywords) (PyObject *obj, PyObject *, const char *, char **, ...);
+extern PyObject * (*gdbpy_BuildValue) (const char *, ...);
+extern PyObject * (*gdbpy_StringFromFormat) (const char *, ...);
+extern PyObject * (*gdbpy_PySequence_Concat) (PyObject *o1, PyObject *o2);
+extern Py_ssize_t (*gdbpy_PyInt_AsSize_t) (PyObject *);
+extern PyObject * (*gdbpy_PyInt_FromSize_t) (size_t size);
+extern PyObject * (*gdbpy_PyInt_FromLong) (long val);
+extern PY_LONG_LONG (*gdbpy_Long_AsLong) (PyObject *);
+extern int (*gdbpy_Dict_SetItem) (PyObject *mp, PyObject *key, PyObject *item);
+extern PyObject * (*gdbpy_Dict_Keys) (PyObject *mp);
+extern void (*gdbpy_Err_SetNone) (PyObject *);
+extern int (*gdbpy_PyObject__IsInstance) (PyObject *object, PyObject *typeorclass);
+extern PyObject * (*gdbpy_PyString_FromStringAndSize) (const char *str, Py_ssize_t size);
+extern PyObject * (*gdbpy_PyBytes_FromStringAndSize) (const char *str, Py_ssize_t size);
+extern PyObject * (*gdbpy_PyMemoryView_FromObject) (PyObject *);
+extern PyObject * (*gdbpy_PyBuffer_FromObject) (PyObject *base, Py_ssize_t offset, Py_ssize_t size);
+extern void (*gdbpy_PyBuffer_Release) (Py_buffer *buf);
+extern int (*gdbpy_PyErr_GivenExceptionMatches) (PyObject *, PyObject *);
+#ifdef IS_PY3K
+extern int (*gdbpy_PyBuffer_FillInfo) (Py_buffer *view, PyObject *exporter, void *buf, Py_ssize_t len, int readonly, int flags);
+extern void (*gdbpy_SetProgramName) (wchar_t *);
+#else
+extern void (*gdbpy_SetProgramName) (char *);
+#endif
+
+#ifdef HAVE_LONG_LONG
+
+extern PyObject * (*gdbpy_PyLong_FromLongLong) (long long val);
+extern PyObject * (*gdbpy_PyLong_FromUnsignedLongLong) (unsigned long long val);
+extern unsigned long long (*gdbpy_PyLong_AsUnsignedLongLong) (PyObject *obj);
+
+#else /* HAVE_LONG_LONG */
+
+extern PyObject * (*gdbpy_PyLong_FromLongLong) (long val);
+extern PyObject * (*gdbpy_PyLong_FromUnsignedLongLong) (unsigned long val);
+extern unsigned long (*gdbpy_PyLong_AsUnsignedLongLong) (PyObject *obj);
+
+#endif /* HAVE_LONG_LONG */
+
+#if defined (IS_PY3K)
+extern int (*gdbpy_PySlice_GetIndicesEx) (PyObject *slice, Py_ssize_t length, Py_ssize_t *start, Py_ssize_t *stop, Py_ssize_t *step, Py_ssize_t *slicelength);
+#else
+extern int (*gdbpy_PySlice_GetIndicesEx) (PySliceObject *slice, Py_ssize_t length, Py_ssize_t *start, Py_ssize_t *stop, Py_ssize_t *step, Py_ssize_t *slicelength);
+#endif
+
+extern int (*gdbpy_PySlice_Check) (PyObject *);
+
+/* Added for dynamic python library loading support */
+#define gdbpy_False gdbpy_Zero
+extern PyObject *gdbpy_None;
+extern PyObject *gdbpy_True;
+extern PyObject *gdbpy_Zero;
+extern PyObject *gdbpy_NotImplemented;
+extern PyTypeObject *gdbpy_FloatType;
+extern PyTypeObject *gdbpy_BoolType;
+extern PyTypeObject *gdbpy_IntType;
+extern PyTypeObject *gdbpy_LongType;
+extern PyTypeObject *gdbpy_SliceType;
+extern PyTypeObject *gdbpy_StringType;
+extern PyTypeObject *gdbpy_ListType;
+extern PyTypeObject *gdbpy_TupleType;
+extern PyTypeObject *gdbpy_UnicodeType;
+
+extern PyObject **pgdbpyExc_AttributeError;
+extern PyObject **pgdbpyExc_IndexError;
+extern PyObject **pgdbpyExc_IOError;
+extern PyObject **pgdbpyExc_KeyError;
+extern PyObject **pgdbpyExc_KeyboardInterrupt;
+extern PyObject **pgdbpyExc_MemoryError;
+extern PyObject **pgdbpyExc_NotImplementedError;
+extern PyObject **pgdbpyExc_OverflowError;
+extern PyObject **pgdbpyExc_RuntimeError;
+extern PyObject **pgdbpyExc_StopIteration;
+extern PyObject **pgdbpyExc_SystemError;
+extern PyObject **pgdbpyExc_TypeError;
+extern PyObject **pgdbpyExc_ValueError;
+extern PyObject **pgdbpyExc_NameError;
+extern PyThreadState **pgdbpy_OSReadlineTState;
+
+extern char * (**pgdbpyOS_ReadlineFunctionPointer) (FILE *, FILE *,
+#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION >= 4
+                        const char *);
+#else
+                        char *);
+#endif
+
+#define gdbpyExc_AttributeError (*pgdbpyExc_AttributeError)
+#define gdbpyExc_IOError (*pgdbpyExc_IOError)
+#define gdbpyExc_KeyError (*pgdbpyExc_KeyError)
+#define gdbpyExc_KeyboardInterrupt (*pgdbpyExc_KeyboardInterrupt)
+#define gdbpyExc_MemoryError (*pgdbpyExc_MemoryError)
+#define gdbpyExc_NotImplementedError (*pgdbpyExc_NotImplementedError)
+#define gdbpyExc_OverflowError (*pgdbpyExc_OverflowError)
+#define gdbpyExc_RuntimeError (*pgdbpyExc_RuntimeError)
+#define gdbpyOS_ReadlineFunctionPointer (*pgdbpyOS_ReadlineFunctionPointer)
+#define gdbpyExc_StopIteration (*pgdbpyExc_StopIteration)
+#define gdbpyExc_SystemError (*pgdbpyExc_SystemError)
+#define gdbpyExc_TypeError (*pgdbpyExc_TypeError)
+#define gdbpyExc_ValueError (*pgdbpyExc_ValueError)
+#define gdbpyExc_NameError (*pgdbpyExc_NameError)
+#define gdbpy_OSReadlineTState (*pgdbpy_OSReadlineTState)
+#define gdbpy_IntCheck(op) PyObject_TypeCheck(op, gdbpy_IntType)
+#define gdbpy_LongCheck(op) PyObject_TypeCheck(op, gdbpy_LongType)
+#define gdbpy_TupleCheck(op) PyObject_TypeCheck(op, gdbpy_TupleType)
+#define gdbpy_StringCheck(op) PyObject_TypeCheck(op, gdbpy_StringType)
+#define gdbpy_UnicodeCheck(op) PyObject_TypeCheck(op, gdbpy_UnicodeType)
+#define gdbpy_ListCheck(op) PyObject_TypeCheck(op, gdbpy_ListType)
+
+/* Macro for returning standard types from a function */
+#define GDB_PY_RETURN_NONE  return Py_INCREF(gdbpy_None), gdbpy_None
+#define GDB_PY_RETURN_TRUE  return Py_INCREF(gdbpy_True), gdbpy_True
+#define GDB_PY_RETURN_FALSE return Py_INCREF(gdbpy_Zero), gdbpy_Zero
+
+/* CUDA-specific defines - end. */
+
+/* The 'name' parameter of PyErr_NewException was missing the 'const'
+   qualifier in Python <= 3.4.  Hence, we wrap it in a function to
+   avoid errors when compiled with -Werror.  */
+
+static inline PyObject*
+gdb_PyErr_NewException (const char *name, PyObject *base, PyObject *dict)
+{
+  return gdbpy_PyErr_NewException (const_cast<char *> (name), base, dict);
+}
+
+#define PyErr_NewException gdb_PyErr_NewException
+
+/* PySys_GetObject's 'name' parameter was missing the 'const'
+   qualifier before Python 3.4.  Hence, we wrap it in a function to
+   avoid errors when compiled with -Werror.  */
+
+static inline PyObject *
+gdb_PySys_GetObject (const char *name)
+{
+  return gdbpy_PySys_GetObject (const_cast<char *> (name));
+}
+
+#define PySys_GetObject gdb_PySys_GetObject
+
+/* PySys_SetPath's 'path' parameter was missing the 'const' qualifier
+   before Python 3.6.  Hence, we wrap it in a function to avoid errors
+   when compiled with -Werror.  */
+
+#ifdef IS_PY3K
+# define GDB_PYSYS_SETPATH_CHAR wchar_t
+#else
+# define GDB_PYSYS_SETPATH_CHAR char
+#endif
+
+extern void (*gdbpy_PySys_SetPath) (GDB_PYSYS_SETPATH_CHAR *path);
+
+static inline void
+gdb_PySys_SetPath (const GDB_PYSYS_SETPATH_CHAR *path)
+{
+  gdbpy_PySys_SetPath (const_cast<GDB_PYSYS_SETPATH_CHAR *> (path));
+}
+
+#define PySys_SetPath gdb_PySys_SetPath
+
+/* Wrap PyGetSetDef to allow convenient construction with string
+   literals.  Unfortunately, PyGetSetDef's 'name' and 'doc' members
+   are 'char *' instead of 'const char *', meaning that in order to
+   list-initialize PyGetSetDef arrays with string literals (and
+   without the wrapping below) would require writing explicit 'char *'
+   casts.  Instead, we extend PyGetSetDef and add constexpr
+   constructors that accept const 'name' and 'doc', hiding the ugly
+   casts here in a single place.  */
+
+struct gdb_PyGetSetDef : PyGetSetDef
+{
+  constexpr gdb_PyGetSetDef (const char *name_, getter get_, setter set_,
+			     const char *doc_, void *closure_)
+    : PyGetSetDef {const_cast<char *> (name_), get_, set_,
+		   const_cast<char *> (doc_), closure_}
+  {}
+
+  /* Alternative constructor that allows omitting the closure in list
+     initialization.  */
+  constexpr gdb_PyGetSetDef (const char *name_, getter get_, setter set_,
+			     const char *doc_)
+    : gdb_PyGetSetDef {name_, get_, set_, doc_, NULL}
+  {}
+
+  /* Constructor for the sentinel entries.  */
+  constexpr gdb_PyGetSetDef (std::nullptr_t)
+    : gdb_PyGetSetDef {NULL, NULL, NULL, NULL, NULL}
+  {}
+};
+
+/* The 'keywords' parameter of PyArg_ParseTupleAndKeywords has type
+   'char **'.  However, string literals are const in C++, and so to
+   avoid casting at every keyword array definition, we'll need to make
+   the keywords array an array of 'const char *'.  To avoid having all
+   callers add a 'const_cast<char **>' themselves when passing such an
+   array through 'char **', we define our own version of
+   PyArg_ParseTupleAndKeywords here with a corresponding 'keywords'
+   parameter type that does the cast in a single place.  (This is not
+   an overload of PyArg_ParseTupleAndKeywords in order to make it
+   clearer that we're calling our own function instead of a function
+   that exists in some newer Python version.)  */
+
+static inline int
+gdb_PyArg_ParseTupleAndKeywords (PyObject *args, PyObject *kw,
+				 const char *format, const char **keywords, ...)
+{
+  va_list ap;
+  int res;
+
+  va_start (ap, keywords);
+  res = gdbpy_PyArg_VaParseTupleAndKeywords (args, kw, format,
+				       const_cast<char **> (keywords),
+				       ap);
+  va_end (ap);
+
+  return res;
+}
+
 /* In order to be able to parse symtab_and_line_to_sal_object function
    a real symtab_and_line structure is needed.  */
 #include "symtab.h"
@@ -217,12 +449,12 @@ extern PyTypeObject symbol_object_type
     CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("symbol_object");
 extern PyTypeObject event_object_type
     CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("event_object");
-extern PyTypeObject stop_event_object_type
-    CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("event_object");
 extern PyTypeObject breakpoint_object_type
     CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("breakpoint_object");
 extern PyTypeObject frame_object_type
     CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("frame_object");
+extern PyTypeObject thread_object_type
+    CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("thread_object");
 
 typedef struct gdbpy_breakpoint_object
 {
@@ -244,7 +476,7 @@ typedef struct gdbpy_breakpoint_object
 #define BPPY_REQUIRE_VALID(Breakpoint)                                  \
     do {                                                                \
       if ((Breakpoint)->bp == NULL)                                     \
-        return gdbpy_ErrFormat (gdbpyExc_RuntimeError,                        \
+        return gdbpy_ErrFormat (gdbpyExc_RuntimeError,                  \
                              _("Breakpoint %d is invalid."),            \
                              (Breakpoint)->number);                     \
     } while (0)
@@ -278,6 +510,8 @@ typedef struct
   PyObject *inf_obj;
 } thread_object;
 
+struct inferior_object;
+
 extern struct cmd_list_element *set_python_list;
 extern struct cmd_list_element *show_python_list;
 
@@ -289,15 +523,14 @@ extern int gdbpy_auto_load_enabled (const struct extension_language_defn *);
 
 extern enum ext_lang_rc gdbpy_apply_val_pretty_printer
   (const struct extension_language_defn *,
-   struct type *type, const gdb_byte *valaddr,
-   LONGEST embedded_offset, CORE_ADDR address,
+   struct value *value,
    struct ui_file *stream, int recurse,
-   const struct value *val,
    const struct value_print_options *options,
    const struct language_defn *language);
 extern enum ext_lang_bt_status gdbpy_apply_frame_filter
   (const struct extension_language_defn *,
-   struct frame_info *frame, int flags, enum ext_lang_frame_args args_type,
+   struct frame_info *frame, frame_filter_flags flags,
+   enum ext_lang_frame_args args_type,
    struct ui_out *out, int frame_low, int frame_high);
 extern void gdbpy_preserve_values (const struct extension_language_defn *,
 				   struct objfile *objfile,
@@ -307,38 +540,29 @@ extern enum ext_lang_bp_stop gdbpy_breakpoint_cond_says_stop
 extern int gdbpy_breakpoint_has_cond (const struct extension_language_defn *,
 				      struct breakpoint *b);
 
-extern void *gdbpy_clone_xmethod_worker_data
-  (const struct extension_language_defn *extlang, void *data);
-extern void gdbpy_free_xmethod_worker_data
-  (const struct extension_language_defn *extlang, void *data);
 extern enum ext_lang_rc gdbpy_get_matching_xmethod_workers
   (const struct extension_language_defn *extlang,
    struct type *obj_type, const char *method_name,
-   xmethod_worker_vec **dm_vec);
-extern enum ext_lang_rc gdbpy_get_xmethod_arg_types
-  (const struct extension_language_defn *extlang,
-   struct xmethod_worker *worker,
-   int *nargs,
-   struct type ***arg_types);
-extern enum ext_lang_rc gdbpy_get_xmethod_result_type
-  (const struct extension_language_defn *extlang,
-   struct xmethod_worker *worker,
-   struct value *object, struct value **args, int nargs,
-   struct type **result_type);
-extern struct value *gdbpy_invoke_xmethod
-  (const struct extension_language_defn *extlang,
-   struct xmethod_worker *worker,
-   struct value *obj, struct value **args, int nargs);
+   std::vector<xmethod_worker_up> *dm_vec);
+
 
 PyObject *gdbpy_history (PyObject *self, PyObject *args);
+PyObject *gdbpy_convenience_variable (PyObject *self, PyObject *args);
+PyObject *gdbpy_set_convenience_variable (PyObject *self, PyObject *args);
 PyObject *gdbpy_breakpoints (PyObject *, PyObject *);
 PyObject *gdbpy_frame_stop_reason_string (PyObject *, PyObject *);
 PyObject *gdbpy_lookup_symbol (PyObject *self, PyObject *args, PyObject *kw);
 PyObject *gdbpy_lookup_global_symbol (PyObject *self, PyObject *args,
 				      PyObject *kw);
+PyObject *gdbpy_lookup_static_symbol (PyObject *self, PyObject *args,
+				      PyObject *kw);
+PyObject *gdbpy_lookup_static_symbols (PyObject *self, PyObject *args,
+					   PyObject *kw);
+PyObject *gdbpy_start_recording (PyObject *self, PyObject *args);
+PyObject *gdbpy_current_recording (PyObject *self, PyObject *args);
+PyObject *gdbpy_stop_recording (PyObject *self, PyObject *args);
 PyObject *gdbpy_newest_frame (PyObject *self, PyObject *args);
 PyObject *gdbpy_selected_frame (PyObject *self, PyObject *args);
-PyObject *gdbpy_block_for_pc (PyObject *self, PyObject *args);
 PyObject *gdbpy_lookup_type (PyObject *self, PyObject *args, PyObject *kw);
 int gdbpy_is_field (PyObject *obj);
 PyObject *gdbpy_create_lazy_string_object (CORE_ADDR address, long length,
@@ -353,6 +577,8 @@ PyObject *gdbpy_parameter_value (enum var_types type, void *var);
 char *gdbpy_parse_command_name (const char *name,
 				struct cmd_list_element ***base_list,
 				struct cmd_list_element **start_list);
+PyObject *gdbpy_register_tui_window (PyObject *self, PyObject *args,
+				     PyObject *kw);
 
 PyObject *symtab_and_line_to_sal_object (struct symtab_and_line sal);
 PyObject *symtab_to_symtab_object (struct symtab *symtab);
@@ -360,18 +586,17 @@ PyObject *symbol_to_symbol_object (struct symbol *sym);
 PyObject *block_to_block_object (const struct block *block,
 				 struct objfile *objfile);
 PyObject *value_to_value_object (struct value *v);
+PyObject *value_to_value_object_no_release (struct value *v);
 PyObject *type_to_type_object (struct type *);
 PyObject *frame_info_to_frame_object (struct frame_info *frame);
 PyObject *symtab_to_linetable_object (PyObject *symtab);
-PyObject *pspace_to_pspace_object (struct program_space *)
-    CPYCHECKER_RETURNS_BORROWED_REF;
+gdbpy_ref<> pspace_to_pspace_object (struct program_space *);
 PyObject *pspy_get_printers (PyObject *, void *);
 PyObject *pspy_get_frame_filters (PyObject *, void *);
 PyObject *pspy_get_frame_unwinders (PyObject *, void *);
 PyObject *pspy_get_xmethods (PyObject *, void *);
 
-PyObject *objfile_to_objfile_object (struct objfile *)
-    CPYCHECKER_RETURNS_BORROWED_REF;
+gdbpy_ref<> objfile_to_objfile_object (struct objfile *);
 PyObject *objfpy_get_printers (PyObject *, void *);
 PyObject *objfpy_get_frame_filters (PyObject *, void *);
 PyObject *objfpy_get_frame_unwinders (PyObject *, void *);
@@ -380,11 +605,13 @@ PyObject *gdbpy_lookup_objfile (PyObject *self, PyObject *args, PyObject *kw);
 
 PyObject *gdbarch_to_arch_object (struct gdbarch *gdbarch);
 
-thread_object *create_thread_object (struct thread_info *tp);
-thread_object *find_thread_object (ptid_t ptid)
-    CPYCHECKER_RETURNS_BORROWED_REF;
-PyObject *find_inferior_object (int pid);
-PyObject *inferior_to_inferior_object (struct inferior *inferior);
+PyObject *gdbpy_new_register_descriptor_iterator (struct gdbarch *gdbarch,
+						  const char *group_name);
+PyObject *gdbpy_new_reggroup_iterator (struct gdbarch *gdbarch);
+
+gdbpy_ref<thread_object> create_thread_object (struct thread_info *tp);
+gdbpy_ref<> thread_to_thread_object (thread_info *thr);;
+gdbpy_ref<inferior_object> inferior_to_inferior_object (inferior *inf);
 
 const struct block *block_object_to_block (PyObject *obj);
 struct symbol *symbol_object_to_symbol (PyObject *obj);
@@ -403,6 +630,12 @@ int gdbpy_initialize_values (void)
   CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
 int gdbpy_initialize_frames (void)
   CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
+int gdbpy_initialize_instruction (void)
+  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
+int gdbpy_initialize_btrace (void)
+  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
+int gdbpy_initialize_record (void)
+  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
 int gdbpy_initialize_symtabs (void)
   CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
 int gdbpy_initialize_commands (void)
@@ -441,42 +674,134 @@ int gdbpy_initialize_event (void)
   CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
 int gdbpy_initialize_py_events (void)
   CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_stop_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_signal_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_breakpoint_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_continue_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_inferior_call_pre_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_inferior_call_post_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_register_changed_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_memory_changed_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_exited_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_thread_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_new_objfile_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_clear_objfiles_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
 int gdbpy_initialize_arch (void)
   CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
+int gdbpy_initialize_registers ()
+  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
 int gdbpy_initialize_xmethods (void)
   CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
 int gdbpy_initialize_unwind (void)
   CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
+int gdbpy_initialize_tui ()
+  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
 
-struct cleanup *make_cleanup_py_decref (PyObject *py);
-struct cleanup *make_cleanup_py_xdecref (PyObject *py);
+/* A wrapper for PyErr_Fetch that handles reference counting for the
+   caller.  */
+class gdbpy_err_fetch
+{
+public:
 
-struct cleanup *ensure_python_env (struct gdbarch *gdbarch,
-				   const struct language_defn *language);
+  gdbpy_err_fetch ()
+  {
+    PyErr_Fetch (&m_error_type, &m_error_value, &m_error_traceback);
+  }
+
+  ~gdbpy_err_fetch ()
+  {
+    Py_XDECREF (m_error_type);
+    Py_XDECREF (m_error_value);
+    Py_XDECREF (m_error_traceback);
+  }
+
+  /* Call PyErr_Restore using the values stashed in this object.
+     After this call, this object is invalid and neither the to_string
+     nor restore methods may be used again.  */
+
+  void restore ()
+  {
+    PyErr_Restore (m_error_type, m_error_value, m_error_traceback);
+    m_error_type = nullptr;
+    m_error_value = nullptr;
+    m_error_traceback = nullptr;
+  }
+
+  /* Return the string representation of the exception represented by
+     this object.  If the result is NULL a python error occurred, the
+     caller must clear it.  */
+
+  gdb::unique_xmalloc_ptr<char> to_string () const;
+
+  /* Return the string representation of the type of the exception
+     represented by this object.  If the result is NULL a python error
+     occurred, the caller must clear it.  */
+
+  gdb::unique_xmalloc_ptr<char> type_to_string () const;
+
+  /* Return true if the stored type matches TYPE, false otherwise.  */
+
+  bool type_matches (PyObject *type) const
+  {
+    return PyErr_GivenExceptionMatches (m_error_type, type);
+  }
+
+private:
+
+  PyObject *m_error_type, *m_error_value, *m_error_traceback;
+};
+
+/* Called before entering the Python interpreter to install the
+   current language and architecture to be used for Python values.
+   Also set the active extension language for GDB so that SIGINT's
+   are directed our way, and if necessary install the right SIGINT
+   handler.  */
+class gdbpy_enter
+{
+ public:
+
+  gdbpy_enter (struct gdbarch *gdbarch, const struct language_defn *language);
+
+  ~gdbpy_enter ();
+
+  DISABLE_COPY_AND_ASSIGN (gdbpy_enter);
+
+ private:
+
+  struct active_ext_lang_state *m_previous_active;
+  PyGILState_STATE m_state;
+  struct gdbarch *m_gdbarch;
+  const struct language_defn *m_language;
+
+  /* An optional is used here because we don't want to call
+     PyErr_Fetch too early.  */
+  gdb::optional<gdbpy_err_fetch> m_error;
+};
+
+/* Like gdbpy_enter, but takes a varobj.  This is a subclass just to
+   make constructor delegation a little nicer.  */
+class gdbpy_enter_varobj : public gdbpy_enter
+{
+ public:
+
+  /* This is defined in varobj.c, where it can access varobj
+     internals.  */
+  gdbpy_enter_varobj (const struct varobj *var);
+
+};
+
+/* The opposite of gdb_enter: this releases the GIL around a region,
+   allowing other Python threads to run.  No Python APIs may be used
+   while this is active.  */
+class gdbpy_allow_threads
+{
+public:
+
+  gdbpy_allow_threads ()
+    : m_save (PyEval_SaveThread ())
+  {
+    gdb_assert (m_save != nullptr);
+  }
+
+  ~gdbpy_allow_threads ()
+  {
+    PyEval_RestoreThread (m_save);
+  }
+
+  DISABLE_COPY_AND_ASSIGN (gdbpy_allow_threads);
+
+private:
+
+  PyThreadState *m_save;
+};
 
 extern struct gdbarch *python_gdbarch;
 extern const struct language_defn *python_language;
@@ -505,31 +830,33 @@ extern const struct language_defn *python_language;
 
 int gdbpy_print_python_errors_p (void);
 void gdbpy_print_stack (void);
+void gdbpy_print_stack_or_quit ();
+void gdbpy_handle_exception () ATTRIBUTE_NORETURN;
 
-PyObject *python_string_to_unicode (PyObject *obj);
-char *unicode_to_target_string (PyObject *unicode_str);
-char *python_string_to_target_string (PyObject *obj);
-PyObject *python_string_to_target_python_string (PyObject *obj);
-char *python_string_to_host_string (PyObject *obj);
-PyObject *host_string_to_python_string (const char *str);
+gdbpy_ref<> python_string_to_unicode (PyObject *obj);
+gdb::unique_xmalloc_ptr<char> unicode_to_target_string (PyObject *unicode_str);
+gdb::unique_xmalloc_ptr<char> python_string_to_target_string (PyObject *obj);
+gdbpy_ref<> python_string_to_target_python_string (PyObject *obj);
+gdb::unique_xmalloc_ptr<char> python_string_to_host_string (PyObject *obj);
+gdbpy_ref<> host_string_to_python_string (const char *str);
 int gdbpy_is_string (PyObject *obj);
-char *gdbpy_obj_to_string (PyObject *obj);
-char *gdbpy_exception_to_string (PyObject *ptype, PyObject *pvalue);
+gdb::unique_xmalloc_ptr<char> gdbpy_obj_to_string (PyObject *obj);
 
 int gdbpy_is_lazy_string (PyObject *result);
 void gdbpy_extract_lazy_string (PyObject *string, CORE_ADDR *addr,
 				struct type **str_type,
-				long *length, char **encoding);
+				long *length,
+				gdb::unique_xmalloc_ptr<char> *encoding);
 
 int gdbpy_is_value_object (PyObject *obj);
 
 /* Note that these are declared here, and not in python.h with the
    other pretty-printer functions, because they refer to PyObject.  */
-PyObject *apply_varobj_pretty_printer (PyObject *print_obj,
-				       struct value **replacement,
-				       struct ui_file *stream);
-PyObject *gdbpy_get_varobj_pretty_printer (struct value *value);
-char *gdbpy_get_display_hint (PyObject *printer);
+gdbpy_ref<> apply_varobj_pretty_printer (PyObject *print_obj,
+					 struct value **replacement,
+					 struct ui_file *stream);
+gdbpy_ref<> gdbpy_get_varobj_pretty_printer (struct value *value);
+gdb::unique_xmalloc_ptr<char> gdbpy_get_display_hint (PyObject *printer);
 PyObject *gdbpy_default_visualizer (PyObject *self, PyObject *args);
 
 void bpfinishpy_pre_stop_hook (struct gdbpy_breakpoint_object *bp_obj);
@@ -547,14 +874,14 @@ extern PyObject *gdbpy_gdb_error;
 extern PyObject *gdbpy_gdb_memory_error;
 extern PyObject *gdbpy_gdberror_exc;
 
-extern void gdbpy_convert_exception (struct gdb_exception)
+extern void gdbpy_convert_exception (const struct gdb_exception &)
     CPYCHECKER_SETS_EXCEPTION;
 
 int get_addr_from_python (PyObject *obj, CORE_ADDR *addr)
     CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
 
-PyObject *gdb_py_object_from_longest (LONGEST l);
-PyObject *gdb_py_object_from_ulongest (ULONGEST l);
+gdbpy_ref<> gdb_py_object_from_longest (LONGEST l);
+gdbpy_ref<> gdb_py_object_from_ulongest (ULONGEST l);
 int gdb_py_int_as_long (PyObject *, long *);
 
 PyObject *gdb_py_generic_dict (PyObject *self, void *closure);
@@ -568,90 +895,36 @@ struct varobj;
 struct varobj_iter *py_varobj_get_iterator (struct varobj *var,
 					    PyObject *printer);
 
-/* CUDA-specific defines.  */
+/* Deleter for Py_buffer unique_ptr specialization.  */
 
-extern int (*gdbpy_Arg_UnpackTuple) (PyObject *, const char *, Py_ssize_t, Py_ssize_t, ...);
-extern PyObject * (*_gdbpy_BuildValue_SizeT) (const char *, ...);
-extern PyObject * (*gdbpy_ErrFormat)(PyObject *, const char *, ...);
-extern PyObject * (*gdbpy_ObjectCallFunctionObjArgs) (PyObject *,...);
-extern PyObject * (*gdbpy_ObjectCallMethodObjArgs) (PyObject *, PyObject *,...);
-extern PyObject * (*gdbpy_ObjectCallMethod)(PyObject *o, char *m, char *format, ...);
+struct Py_buffer_deleter
+{
+  void operator() (Py_buffer *b) const
+  {
+    gdbpy_PyBuffer_Release (b);
+  }
+};
 
-extern int (*gdbpy_ArgParseTuple) (PyObject *, const char *, ...);
-extern int (*gdbpy_ArgParseTupleAndKeywords) (PyObject *obj, PyObject *, const char *, char **, ...);
-extern PyObject * (*gdbpy_BuildValue) (const char *, ...);
-extern PyObject * (*gdbpy_StringFromFormat) (const char *, ...);
-extern PyObject * (*gdbpy_Sequence_Concat) (PyObject *o1, PyObject *o2);
-extern PY_LONG_LONG (*gdbpy_Long_AsLong) (PyObject *);
-extern int (*gdbpy_Dict_SetItem) (PyObject *mp, PyObject *key, PyObject *item);
-extern PyObject * (*gdbpy_Dict_Keys) (PyObject *mp);
-extern void (*gdbpy_Err_SetNone) (PyObject *);
-extern int (*gdbpy_Object_IsInstance) (PyObject *object, PyObject *typeorclass);
+/* A unique_ptr specialization for Py_buffer.  */
+typedef std::unique_ptr<Py_buffer, Py_buffer_deleter> Py_buffer_up;
 
-/* Added for dynamic python library loading support */
-#define gdbpy_False gdbpy_Zero
-extern PyObject *gdbpy_None;
-extern PyObject *gdbpy_True;
-extern PyObject *gdbpy_Zero;
-extern PyObject *gdbpy_NotImplemented;
-extern PyTypeObject *gdbpy_FloatType;
-extern PyTypeObject *gdbpy_BoolType;
-extern PyTypeObject *gdbpy_IntType;
-extern PyTypeObject *gdbpy_LongType;
-extern PyTypeObject *gdbpy_StringType;
-extern PyTypeObject *gdbpy_ListType;
-extern PyTypeObject *gdbpy_TupleType;
-extern PyTypeObject *gdbpy_UnicodeType;
+/* Parse a register number from PYO_REG_ID and place the register number
+   into *REG_NUM.  The register is a register for GDBARCH.
 
-extern PyObject **pgdbpyExc_AttributeError;
-extern PyObject **pgdbpyExc_IOError;
-extern PyObject **pgdbpyExc_KeyError;
-extern PyObject **pgdbpyExc_KeyboardInterrupt;
-extern PyObject **pgdbpyExc_MemoryError;
-extern PyObject **pgdbpyExc_NotImplementedError;
-extern PyObject **pgdbpyExc_OverflowError;
-extern PyObject **pgdbpyExc_RuntimeError;
-extern PyObject **pgdbpyExc_StopIteration;
-extern PyObject **pgdbpyExc_SystemError;
-extern PyObject **pgdbpyExc_TypeError;
-extern PyObject **pgdbpyExc_ValueError;
-extern PyObject **pgdbpyExc_NameError;
-extern PyThreadState **pgdbpy_OSReadlineTState;
+   If a register is parsed successfully then *REG_NUM will have been
+   updated, and true is returned.  Otherwise the contents of *REG_NUM are
+   undefined, and false is returned.
 
-extern char * (**pgdbpyOS_ReadlineFunctionPointer) (FILE *, FILE *,
-#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION >= 4
-			const char *);
-#else
-			char *);
-#endif
-extern PyThreadState **gdbpy_ThreadState_Current;
+   The PYO_REG_ID object can be a string, the name of the register.  This
+   is the slowest approach as GDB has to map the name to a number for each
+   call.  Alternatively PYO_REG_ID can be an internal GDB register
+   number.  This is quick but should not be encouraged as this means
+   Python scripts are now dependent on GDB's internal register numbering.
+   Final PYO_REG_ID can be a gdb.RegisterDescriptor object, these objects
+   can be looked up by name once, and then cache the register number so
+   should be as quick as using a register number.  */
 
-#define gdbpyExc_AttributeError (*pgdbpyExc_AttributeError)
-#define gdbpyExc_IOError (*pgdbpyExc_IOError)
-#define gdbpyExc_KeyError (*pgdbpyExc_KeyError)
-#define gdbpyExc_KeyboardInterrupt (*pgdbpyExc_KeyboardInterrupt)
-#define gdbpyExc_MemoryError (*pgdbpyExc_MemoryError)
-#define gdbpyExc_NotImplementedError (*pgdbpyExc_NotImplementedError)
-#define gdbpyExc_OverflowError (*pgdbpyExc_OverflowError)
-#define gdbpyExc_RuntimeError (*pgdbpyExc_RuntimeError)
-#define gdbpyOS_ReadlineFunctionPointer (*pgdbpyOS_ReadlineFunctionPointer)
-#define gdbpyExc_StopIteration (*pgdbpyExc_StopIteration)
-#define gdbpyExc_SystemError (*pgdbpyExc_SystemError)
-#define gdbpyExc_TypeError (*pgdbpyExc_TypeError)
-#define gdbpyExc_ValueError (*pgdbpyExc_ValueError)
-#define gdbpyExc_NameError (*pgdbpyExc_NameError)
-#define gdbpy_OSReadlineTState (*pgdbpy_OSReadlineTState)
-#define gdbpy_IntCheck(op) PyObject_TypeCheck(op, gdbpy_IntType)
-#define gdbpy_LongCheck(op) PyObject_TypeCheck(op, gdbpy_LongType)
-#define gdbpy_TupleCheck(op) PyObject_TypeCheck(op, gdbpy_TupleType)
-#define gdbpy_StringCheck(op) PyObject_TypeCheck(op, gdbpy_StringType)
-#define gdbpy_UnicodeCheck(op) PyObject_TypeCheck(op, gdbpy_UnicodeType)
-#define gdbpy_ListCheck(op) PyObject_TypeCheck(op, gdbpy_ListType)
+extern bool gdbpy_parse_register_id (struct gdbarch *gdbarch,
+				     PyObject *pyo_reg_id, int *reg_num);
 
-/* Macro for returning standard types from a function */
-#define GDB_PY_RETURN_NONE  return Py_INCREF(gdbpy_None), gdbpy_None
-#define GDB_PY_RETURN_TRUE  return Py_INCREF(gdbpy_True), gdbpy_True
-#define GDB_PY_RETURN_FALSE return Py_INCREF(gdbpy_Zero), gdbpy_Zero
-
-
-#endif /* GDB_PYTHON_INTERNAL_H */
+#endif /* PYTHON_PYTHON_INTERNAL_H */
