@@ -1,5 +1,5 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2017-2019 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2017-2021 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,7 @@
 #define REMOTE_NTO_H
 
 #include "nto-share/dsmsgs.h"
+#include "gdbsupport/byte-vector.h"
 #include <stdbool.h>
 
 typedef union
@@ -28,11 +29,16 @@ typedef union
   TSMsg_text_t text;
 } DScomm_t;
 
+
 /* Compatibility functions for the CUDA remote I/O */
+struct remote_target;
+
 int
-qnx_getpkt (char **buf, long *sizeof_buf, int forever);
+qnx_getpkt (remote_target *ops, gdb::char_vector* buf, int forever);
 int
-qnx_putpkt (const char *buf);
+qnx_getpkt_sane (gdb::char_vector* buf, int forever);
+int
+qnx_putpkt (remote_target *ops, const char *buf);
 int
 qnx_putpkt_binary (const char *buf, int cnt);
 
